/* Copyright (c) 2015, Electric Cloud, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* (As an historical note, the above license instantiates
 * the "BSD 2-Clause License" template that was presented
 * by the Open Source Initiative at
 *
 *   http://www.opensource.org/licenses/bsd-license.php
 *
 * on 2012-02-23.)
 */

/* Summary:
 *
 * This header detects certain architectural details, but makes no interface
 * definitions.  Interface headers may find it convenient to use this header,
 * but their definitions should be viewed as dependent upon the architectural
 * details themselves, rather than some abstraction defined by this header.
 *
 * If desired, interface headers can be rewritten to avoid the use of this
 * header, though often they must detect the architecture in some fashion.
 */

#ifndef VRPSX_ARCH_H
#define VRPSX_ARCH_H 1


#if defined(__GNUC__)

#define VRPSX_GNUC 1

#if defined(__i386__) || defined(__x86_64__)

#define VRPSX_X86 1

#if defined(__x86_64__)

#define VRPSX_WORD_BITS 64
#define VRPSX_ATOMIC_BITS 64

#else /* word size */

#define VRPSX_WORD_BITS 32
#define VRPSX_ATOMIC_BITS 32

#endif /* word size */

#elif defined(__sparc__)

#define VRPSX_SPARC 1

#if defined(__sparcv9)

#define VRPSX_WORD_BITS 64
#define VRPSX_ATOMIC_BITS 64

#elif defined(__sparcv8)

#define VRPSX_WORD_BITS 32
#define VRPSX_ATOMIC_BITS 32

#else /* word size */

#error unrecognized sparc variant

#endif /* word size */

#else /* instruction set */

#error instruction set not yet supported

#endif /* instruction set */

#elif defined(_MSC_VER)

#define VRPSX_MSC 1

#if defined(_M_IX86)

#define VRPSX_X86 1
#define VRPSX_WORD_BITS 32
#define VRPSX_ATOMIC_BITS 32

#elif defined(_M_X64)

#define VRPSX_X86 1
#define VRPSX_WORD_BITS 64
#define VRPSX_ATOMIC_BITS 64

#else /* instruction set */

#error instruction set not yet supported

#endif /* instruction set */

#else /* compiler */

#error compiler not yet supported

#endif /* compiler */


#endif /* VRPSX_ARCH_H */
