/* Copyright (c) 2015, Electric Cloud, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* (As an historical note, the above license instantiates
 * the "BSD 2-Clause License" template that was presented
 * by the Open Source Initiative at
 *
 *   http://www.opensource.org/licenses/bsd-license.php
 *
 * on 2012-02-23.)
 */

/* Summary:
 *
 * An interface to a POSIX-like system error.
 */

#ifndef VRPSX_SYSTEM_ERROR_H
#define VRPSX_SYSTEM_ERROR_H 1

#include "vrpsx_identified.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/* It seems unlikely that even future binary-incompatible variants
 * of this interface would need to depart from "int", but by using
 * this typedef we provide convenient documentation of intent.
 */
typedef int vrpsx_errno_t;

/* An interface to a POSIX-like system error.
 *
 * You can cast a pointer to this interface to (vrpsx_error_t *)
 * because this interface extends vrpsx_error_t.
 *
 * Extends: vrpsx_error_t.
 * Interface Lifetime: reference counted.
 * Thread Safety: thread-safe.
 */
typedef struct vrpsx_system_error_struct {
#define VRPSX_SYSTEM_ERROR_MEMBERS(VTBL)        \
    VRPSX_ERROR_MEMBERS(VTBL)

    VRPSX_SYSTEM_ERROR_MEMBERS(struct vrpsx_system_error_vtbl_struct)
} vrpsx_system_error_t;

typedef struct vrpsx_system_error_end_struct {
} vrpsx_system_error_end_t;

typedef struct vrpsx_system_error_vtbl_struct {
#define VRPSX_SYSTEM_ERROR_VTBL_MEMBERS(TYPE)                           \
    VRPSX_ERROR_VTBL_MEMBERS(TYPE)                                      \
                                                                        \
    /* Returns the POSIX-like error number for this error.              \
     */                                                                 \
    vrpsx_errno_t (*number)(                                            \
        TYPE const *system_error);                                      \
                                                                        \
    /* End marker for documentation and to help detect initialization bugs. \
     */                                                                 \
    vrpsx_system_error_end_t const *system_error_end;

    VRPSX_SYSTEM_ERROR_VTBL_MEMBERS(vrpsx_system_error_t);
} vrpsx_system_error_vtbl_t;


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* VRPSX_SYSTEM_ERROR_H */
